"""
Pterodactyl API Wrapper.

Pterodactyl is an open source game control panel.  Pydactyl is a Python package
that provides a simplified interface to the API.
"""

from .client import PterodactylClient  # noqa
from .constants import __version__  # noqa
